// file      : liblava/base/memory.hpp
// copyright : Copyright (c) 2018-present, Lava Block OÜ and contributors
// license   : MIT; see accompanying LICENSE file

#pragma once

// clang-format off

#include <liblava/base/base.hpp>
#include <vk_mem_alloc.h>

// clang-format on

namespace lava {

    // fwd
    struct device;
    using device_cptr = device const*;

    struct allocator {
        using ptr = std::shared_ptr<allocator>;

        allocator() = default;
        explicit allocator(VmaAllocator allocator)
        : vma_allocator(allocator) {}

        bool create(device_cptr device, VmaAllocatorCreateFlags flags = 0);
        void destroy();

        bool valid() const {
            return vma_allocator != nullptr;
        }

        VmaAllocator get() const {
            return vma_allocator;
        }

    private:
        VmaAllocator vma_allocator = nullptr;
    };

    inline allocator::ptr make_allocator() {
        return std::make_shared<allocator>();
    }

    inline allocator::ptr create_allocator(device_cptr device, VmaAllocatorCreateFlags flags = 0) {
        auto result = make_allocator();
        if (!result->create(device, flags))
            return nullptr;

        return result;
    }

    struct memory : no_copy_no_move {
        static memory& get() {
            static memory memory;
            return memory;
        }

        static VkAllocationCallbacks* alloc() {
            if (get().use_custom_cpu_callbacks)
                return &get().vk_callbacks;

            return nullptr;
        }

        static type find_type_with_properties(VkPhysicalDeviceMemoryProperties properties, ui32 type_bits,
                                              VkMemoryPropertyFlags required_properties);

        static type find_type(VkPhysicalDevice gpu, VkMemoryPropertyFlags properties, ui32 type_bits);

        void set_callbacks(VkAllocationCallbacks const& callbacks) {
            vk_callbacks = callbacks;
        }
        void set_use_custom_cpu_callbacks(bool value) {
            use_custom_cpu_callbacks = value;
        }

    private:
        memory();

        bool use_custom_cpu_callbacks = true;
        VkAllocationCallbacks vk_callbacks = {};
    };

} // namespace lava
