// file      : liblava/core/types.hpp
// copyright : Copyright (c) 2018-present, Lava Block OÜ and contributors
// license   : MIT; see accompanying LICENSE file

#pragma once

#include <cassert>
#include <cstdint>
#include <functional>
#include <liblava/core/def.hpp>
#include <map>
#include <string>
#include <string_view>
#include <vector>

namespace lava {

    using int8 = std::int8_t;
    using i8 = int8;

    using uint8 = std::uint8_t;
    using ui8 = uint8;

    using int16 = std::int16_t;
    using i16 = int16;

    using uint16 = std::uint16_t;
    using ui16 = uint16;

    using int32 = std::int32_t;
    using i32 = int32;

    using uint32 = std::uint32_t;
    using ui32 = uint32;

    using int64 = std::int64_t;
    using i64 = int64;

    using uint64 = std::uint64_t;
    using ui64 = uint64;

    using char8 = std::int_fast8_t;
    using c8 = char8;

    using uchar8 = std::uint_fast8_t;
    using uc8 = uchar8;

    using char16 = int16;
    using c16 = char16;

    using uchar16 = uint16;
    using uc16 = uchar16;

    using char32 = int32;
    using c32 = char32;

    using uchar32 = uint32;
    using uc32 = uchar32;

    using size_t = std::size_t;
    using uchar = unsigned char;
    using r32 = float;
    using r64 = double;
    using real = r64;
    using delta = r32;

    using void_ptr = void*;
    using void_cptr = void const*;

    using type = ui32;
    constexpr type const no_type = 0xffffffff;
    constexpr type const undef = 0;

    using index = type;
    constexpr index const no_index = no_type;
    using index_list = std::vector<index>;
    using index_map = std::map<index, index>;

    using string = std::string;
    using string_ref = string const&;
    using string_list = std::vector<string>;
    using string_view = std::string_view;

    using name = char const*;
    using names = std::vector<name>;
    using names_ref = names const&;

    constexpr name _lava_ = "lava";
    constexpr name _liblava_ = "liblava";

    constexpr name _zip_ = "zip";
    constexpr name _default_ = "default";

    inline name str(string_ref value) {
        return value.c_str();
    }

    template<typename T>
    inline r32 to_r32(T value) {
        return static_cast<r32>(value);
    }

    template<typename T>
    inline r64 to_r64(T value) {
        return static_cast<r64>(value);
    }

    template<typename T>
    inline i32 to_i32(T value) {
        return static_cast<i32>(value);
    }

    template<typename T>
    inline i64 to_i64(T value) {
        return static_cast<i64>(value);
    }

    template<typename T>
    inline ui32 to_ui32(T value) {
        return static_cast<ui32>(value);
    }

    template<typename T>
    inline ui64 to_ui64(T value) {
        return static_cast<ui64>(value);
    }

    template<typename T>
    inline size_t to_size_t(T value) {
        return static_cast<size_t>(value);
    }

    template<typename T>
    inline index to_index(T value) {
        return static_cast<index>(value);
    }

    template<typename T>
    inline char const* icon(T value) {
        return (char const*) value;
    }

    struct no_copy_no_move {
        no_copy_no_move() = default;
        no_copy_no_move(no_copy_no_move const&) = delete;

        void operator=(no_copy_no_move const&) = delete;
    };

    struct interface {
        virtual ~interface() = default;
    };

} // namespace lava
